@@ -1895,3 +1895,55 @@ def test_item_fields_from_custom_view_on_card_internal_id(pub):
     resp = get_app(pub).get('/test/', status=200)
     # check options
     assert resp.form['f1'].options == [('1', False, 'Foo 0'), ('2', False, 'Foo 1')]
+
+
+def test_item_fields_users_datasource_with_role_template(pub):
+    pub.role_class.wipe()
+    role = pub.role_class(name='xxx')
+    role.id = role.uuid = '42'
+    role.store()
+
+    pub.user_class.wipe()
+    user = pub.user_class(name='John Doe')
+    user.roles = [role.id]
+    user.store()
+
+    user = pub.user_class(name='Jane Doe')
+    user.store()
+
+    NamedDataSource.wipe()
+    datasource = NamedDataSource(name='foo')
+    datasource.data_source = {'type': 'wcs:users'}
+    datasource.store()
+
+    FormDef.wipe()
+    formdef = FormDef()
+    formdef.name = 'test'
+    formdef.workflow_roles = {'_receiver': role.id}
+    formdef.fields = [
+        fields.ItemField(id='1', label='User', data_source={'type': 'foo'}),
+    ]
+    formdef.store()
+
+    app = get_app(pub)
+
+    resp = app.get('/test/')
+    assert [x[2] for x in resp.form['f1'].options] == ['Jane Doe', 'John Doe']
+
+    datasource.users_included_roles = ['{{ form_role_receiver_uuid }}']
+    datasource.store()
+
+    resp = app.get('/test/')
+    assert [x[2] for x in resp.form['f1'].options] == ['John Doe']
+
+    resp = resp.form.submit('submit')
+    resp = resp.form.submit('submit').follow()  # validation page
+
+    assert 'John Doe' in resp.text
+
+    datasource.users_included_roles = []
+    datasource.users_excluded_roles = ['{{ form_role_receiver_uuid }}']
+    datasource.store()
+
+    resp = app.get('/test/')
+    assert [x[2] for x in resp.form['f1'].options] == ['Jane Doe']
