@@ -19,6 +19,7 @@ import collections
 import copy
 import datetime
 import decimal
+import email
 import functools
 import itertools
 import json
@@ -37,7 +38,7 @@ from django.utils.timezone import localtime
 from quixote import get_publisher, get_session
 from quixote.html import TemplateIO, htmltext
 
-from wcs.qommon.misc import localstrftime
+from wcs.qommon.misc import localstrftime, simplify
 
 from . import data_sources, fields
 from .categories import Category
@@ -161,6 +162,7 @@ class FormDefBase(StoredObjectMixin):
     confirmation = True
     detailed_emails = True
     disabled = False
+    hidden_from_api_results = False
     only_allow_one = False
     enable_tracking_codes = False
     tracking_code_verify_fields = None
@@ -230,6 +232,7 @@ class FormDefBase(StoredObjectMixin):
         'discussion',
         'detailed_emails',
         'disabled',
+        'hidden_from_api_results',
         'only_allow_one',
         'enable_tracking_codes',
         'confirmation',
@@ -1214,7 +1217,14 @@ class FormDefBase(StoredObjectMixin):
                 d[f'{field.id}'] = d[f'{field.id}_display'] = d[f'{field.id}_structured'] = None
         return d
 
-    def export_to_json(self, include_id=False, indent=None, with_user_fields=False, with_block_schemas=False):
+    def export_to_json(
+        self,
+        include_id=False,
+        indent=None,
+        with_user_fields=False,
+        with_block_schemas=False,
+        with_custom_views=False,
+    ):
         from wcs.carddef import CardDef
 
         root = {}
@@ -1256,6 +1266,11 @@ class FormDefBase(StoredObjectMixin):
             if getattr(self, attr):
                 root[attr] = sorted(getattr(self, attr))
 
+        if with_custom_views:
+            custom_views = get_publisher().custom_view_class.select_shared_for_formdef(self)
+            root['custom_views'] = [{'id': x.slug, 'text': x.title} for x in custom_views]
+            root['custom_views'].sort(key=lambda x: simplify(x['text']))
+
         if isinstance(self, CardDef):
             all_carddefs = CardDef.select(ignore_errors=True)
             all_carddefs = [c for c in all_carddefs if c]
@@ -1515,7 +1530,11 @@ class FormDefBase(StoredObjectMixin):
                 element.attrib['type'] = 'file'
                 ET.SubElement(element, 'filename').text = value.base_filename
                 ET.SubElement(element, 'content_type').text = value.content_type or 'application/octet-stream'
-                ET.SubElement(element, 'content').text = force_str(base64.b64encode(value.get_content()))
+                # use email.base64mime.body_encode to limit line lengths to 76 characters,
+                # so difflib doesn't crash on them in inspect mode.
+                ET.SubElement(element, 'content').text = force_str(
+                    email.base64mime.body_encode(value.get_content())
+                )
             elif isinstance(value, time.struct_time):
                 element.text = time.strftime('%Y-%m-%d', value)
                 element.attrib['type'] = 'date'
@@ -2350,7 +2369,7 @@ class FormDefBase(StoredObjectMixin):
         for field in self.fields or []:
             yield from field.i18n_scan(base_location=location + 'fields/')
 
-    def get_last_test_results(self, extra_criterias=None, order_by='-id'):
+    def get_test_results(self, extra_criterias=None, order_by='-id'):
         from wcs.testdef import TestDef, TestResults
 
         criterias = [Equal('object_type', self.get_table_name()), Equal('object_id', str(self.id))]
@@ -2361,8 +2380,10 @@ class FormDefBase(StoredObjectMixin):
         if extra_criterias:
             criterias.extend(extra_criterias)
 
-        test_results = TestResults.select(criterias, order_by=order_by)
+        return TestResults.select(criterias, order_by=order_by)
 
+    def get_last_test_results(self, extra_criterias=None, order_by='-id'):
+        test_results = self.get_test_results(extra_criterias=extra_criterias, order_by=order_by)
         if not test_results:
             return
 
